#include <OpenSim/Common/SimmSpline.h>
#include <OpenSim/Common/SimmMacros.h>
#include <OpenSim/Simulation/Model/Model.h>
#include <math.h>
#include<iostream>
#include<fstream>
#include <OpenSim/Simulation/Model/GeometryPath.h>
#include <OpenSim/Simulation/Model/PointForceDirection.h>
#include "NonLinearLigament.h"

#define NumBitsPerChar                 8U

/* Function: rtGetNaN ==================================================
 * Abstract:
 * Initialize rtNaN needed by the generated code.
 * NaN is initialized as non-signaling. Assumes IEEE.
 */
real_T rtGetNaN(void)
{
	real_T nan = 0.0;
	uint16_T one = 1U;
	enum {
		LittleEndian,
		BigEndian
	} machByteOrder = (*((uint8_T *)&one) == 1U) ? LittleEndian : BigEndian;
	switch (machByteOrder) {
	case LittleEndian:
	{
		union {
			LittleEndianIEEEDouble bitVal;
			real_T fltVal;
		} tmpVal;

		tmpVal.bitVal.words.wordH = 0xFFF80000U;
		tmpVal.bitVal.words.wordL = 0x00000000U;
		nan = tmpVal.fltVal;
		break;
	}

	case BigEndian:
	{
		union {
			BigEndianIEEEDouble bitVal;
			real_T fltVal;
		} tmpVal;

		tmpVal.bitVal.words.wordH = 0x7FFFFFFFU;
		tmpVal.bitVal.words.wordL = 0xFFFFFFFFU;
		nan = tmpVal.fltVal;
		break;
	}
	}

	return nan;
}

/* Function: rtGetNaNF ==================================================
 * Abstract:
 * Initialize rtNaNF needed by the generated code.
 * NaN is initialized as non-signaling. Assumes IEEE.
 */
real32_T rtGetNaNF(void)
{
	IEEESingle nanF = { { 0 } };

	uint16_T one = 1U;
	enum {
		LittleEndian,
		BigEndian
	} machByteOrder = (*((uint8_T *)&one) == 1U) ? LittleEndian : BigEndian;
	switch (machByteOrder) {
	case LittleEndian:
	{
		nanF.wordL.wordLuint = 0xFFC00000U;
		break;
	}

	case BigEndian:
	{
		nanF.wordL.wordLuint = 0x7FFFFFFFU;
		break;
	}
	}

	return nanF.wordL.wordLreal;
}

/*
 * File trailer for rtGetNaN.cpp
 *
 * [EOF]
 */

#define NumBitsPerChar                 8U

/* Function: rtGetInf ==================================================
 * Abstract:
 * Initialize rtInf needed by the generated code.
 * Inf is initialized as non-signaling. Assumes IEEE.
 */
real_T rtGetInf(void)
{
	real_T inf = 0.0;
	uint16_T one = 1U;
	enum {
		LittleEndian,
		BigEndian
	} machByteOrder = (*((uint8_T *)&one) == 1U) ? LittleEndian : BigEndian;
	switch (machByteOrder) {
	case LittleEndian:
	{
		union {
			LittleEndianIEEEDouble bitVal;
			real_T fltVal;
		} tmpVal;

		tmpVal.bitVal.words.wordH = 0x7FF00000U;
		tmpVal.bitVal.words.wordL = 0x00000000U;
		inf = tmpVal.fltVal;
		break;
	}

	case BigEndian:
	{
		union {
			BigEndianIEEEDouble bitVal;
			real_T fltVal;
		} tmpVal;

		tmpVal.bitVal.words.wordH = 0x7FF00000U;
		tmpVal.bitVal.words.wordL = 0x00000000U;
		inf = tmpVal.fltVal;
		break;
	}
	}

	return inf;
}

/* Function: rtGetInfF ==================================================
 * Abstract:
 * Initialize rtInfF needed by the generated code.
 * Inf is initialized as non-signaling. Assumes IEEE.
 */
real32_T rtGetInfF(void)
{
	IEEESingle infF;
	infF.wordL.wordLuint = 0x7F800000U;
	return infF.wordL.wordLreal;
}

/* Function: rtGetMinusInf ==================================================
 * Abstract:
 * Initialize rtMinusInf needed by the generated code.
 * Inf is initialized as non-signaling. Assumes IEEE.
 */
real_T rtGetMinusInf(void)
{
	real_T minf = 0.0;
	uint16_T one = 1U;
	enum {
		LittleEndian,
		BigEndian
	} machByteOrder = (*((uint8_T *)&one) == 1U) ? LittleEndian : BigEndian;
	switch (machByteOrder) {
	case LittleEndian:
	{
		union {
			LittleEndianIEEEDouble bitVal;
			real_T fltVal;
		} tmpVal;

		tmpVal.bitVal.words.wordH = 0xFFF00000U;
		tmpVal.bitVal.words.wordL = 0x00000000U;
		minf = tmpVal.fltVal;
		break;
	}

	case BigEndian:
	{
		union {
			BigEndianIEEEDouble bitVal;
			real_T fltVal;
		} tmpVal;

		tmpVal.bitVal.words.wordH = 0xFFF00000U;
		tmpVal.bitVal.words.wordL = 0x00000000U;
		minf = tmpVal.fltVal;
		break;
	}
	}

	return minf;
}

/* Function: rtGetMinusInfF ==================================================
 * Abstract:
 * Initialize rtMinusInfF needed by the generated code.
 * Inf is initialized as non-signaling. Assumes IEEE.
 */
real32_T rtGetMinusInfF(void)
{
	IEEESingle minfF;
	minfF.wordL.wordLuint = 0xFF800000U;
	return minfF.wordL.wordLreal;
}

/*
 * File trailer for rtGetInf.cpp
 *
 * [EOF]
 */

real_T rtInf;
real_T rtMinusInf;
real_T rtNaN;
real32_T rtInfF;
real32_T rtMinusInfF;
real32_T rtNaNF;

/* Function: rt_InitInfAndNaN ==================================================
 * Abstract:
 * Initialize the rtInf, rtMinusInf, and rtNaN needed by the
 * generated code. NaN is initialized as non-signaling. Assumes IEEE.
 */
void rt_InitInfAndNaN(size_t realSize)
{
	(void)(realSize);
	rtNaN = rtGetNaN();
	rtNaNF = rtGetNaNF();
	rtInf = rtGetInf();
	rtInfF = rtGetInfF();
	rtMinusInf = rtGetMinusInf();
	rtMinusInfF = rtGetMinusInfF();
}

/* Function: rtIsInf ==================================================
 * Abstract:
 * Test if value is infinite
 */
boolean_T rtIsInf(real_T value)
{
	return ((value == rtInf || value == rtMinusInf) ? 1U : 0U);
}

/* Function: rtIsInfF =================================================
 * Abstract:
 * Test if single-precision value is infinite
 */
boolean_T rtIsInfF(real32_T value)
{
	return(((value) == rtInfF || (value) == rtMinusInfF) ? 1U : 0U);
}

/* Function: rtIsNaN ==================================================
 * Abstract:
 * Test if value is not a number
 */
boolean_T rtIsNaN(real_T value)
{

#if defined(_MSC_VER) && (_MSC_VER <= 1200)

	return _isnan(value) ? TRUE : FALSE;

#else

	return (value != value) ? 1U : 0U;

#endif

}

/* Function: rtIsNaNF =================================================
 * Abstract:
 * Test if single-precision value is not a number
 */
boolean_T rtIsNaNF(real32_T value)
{

#if defined(_MSC_VER) && (_MSC_VER <= 1200)

	return _isnan((real_T)value) ? true : false;

#else

	return (value != value) ? 1U : 0U;

#endif

}

/*
 * File trailer for rt_nonfinite.cpp
 *
 * [EOF]
 */



 // Function Definitions

 //
 // Arguments    : emxArray_real_T *emxArray
 //                int oldNumel
 // Return Type  : void
 //
void emxEnsureCapacity_real_T(emxArray_real_T *emxArray, int oldNumel)
{
	int newNumel;
	int i;
	void *newData;
	if (oldNumel < 0) {
		oldNumel = 0;
	}

	newNumel = 1;
	for (i = 0; i < emxArray->numDimensions; i++) {
		newNumel *= emxArray->size[i];
	}

	if (newNumel > emxArray->allocatedSize) {
		i = emxArray->allocatedSize;
		if (i < 16) {
			i = 16;
		}

		while (i < newNumel) {
			if (i > 1073741823) {
				i = MAX_int32_T;
			}
			else {
				i <<= 1;
			}
		}

		newData = calloc((unsigned int)i, sizeof(double));
		if (emxArray->data != NULL) {
			memcpy(newData, (void *)emxArray->data, sizeof(double) * oldNumel);
			if (emxArray->canFreeData) {
				free((void *)emxArray->data);
			}
		}

		emxArray->data = (double *)newData;
		emxArray->allocatedSize = i;
		emxArray->canFreeData = true;
	}
}

//
// Arguments    : emxArray_real_T *emxArray
//                int oldNumel
// Return Type  : void
//
void emxEnsureCapacity_real_T1(emxArray_real_T *emxArray, int oldNumel)
{
	int newNumel;
	int i;
	void *newData;
	if (oldNumel < 0) {
		oldNumel = 0;
	}

	newNumel = 1;
	for (i = 0; i < emxArray->numDimensions; i++) {
		newNumel *= emxArray->size[i];
	}

	if (newNumel > emxArray->allocatedSize) {
		i = emxArray->allocatedSize;
		if (i < 16) {
			i = 16;
		}

		while (i < newNumel) {
			if (i > 1073741823) {
				i = MAX_int32_T;
			}
			else {
				i <<= 1;
			}
		}

		newData = calloc((unsigned int)i, sizeof(double));
		if (emxArray->data != NULL) {
			memcpy(newData, (void *)emxArray->data, sizeof(double) * oldNumel);
			if (emxArray->canFreeData) {
				free((void *)emxArray->data);
			}
		}

		emxArray->data = (double *)newData;
		emxArray->allocatedSize = i;
		emxArray->canFreeData = true;
	}
}

//
// Arguments    : emxArray_real_T **pEmxArray
// Return Type  : void
//
void emxFree_real_T(emxArray_real_T **pEmxArray)
{
	if (*pEmxArray != (emxArray_real_T *)NULL) {
		if (((*pEmxArray)->data != (double *)NULL) && (*pEmxArray)->canFreeData) {
			free((void *)(*pEmxArray)->data);
		}

		free((void *)(*pEmxArray)->size);
		free((void *)*pEmxArray);
		*pEmxArray = (emxArray_real_T *)NULL;
	}
}

//
// Arguments    : emxArray_real_T **pEmxArray
//                int numDimensions
// Return Type  : void
//
void emxInit_real_T(emxArray_real_T **pEmxArray, int numDimensions)
{
	emxArray_real_T *emxArray;
	int i;
	*pEmxArray = (emxArray_real_T *)malloc(sizeof(emxArray_real_T));
	emxArray = *pEmxArray;
	emxArray->data = (double *)NULL;
	emxArray->numDimensions = numDimensions;
	emxArray->size = (int *)malloc((unsigned int)(sizeof(int) * numDimensions));
	emxArray->allocatedSize = 0;
	emxArray->canFreeData = true;
	for (i = 0; i < numDimensions; i++) {
		emxArray->size[i] = 0;
	}
}

//
// Arguments    : emxArray_real_T **pEmxArray
//                int numDimensions
// Return Type  : void
//
void emxInit_real_T1(emxArray_real_T **pEmxArray, int numDimensions)
{
	emxArray_real_T *emxArray;
	int i;
	*pEmxArray = (emxArray_real_T *)malloc(sizeof(emxArray_real_T));
	emxArray = *pEmxArray;
	emxArray->data = (double *)NULL;
	emxArray->numDimensions = numDimensions;
	emxArray->size = (int *)malloc((unsigned int)(sizeof(int) * numDimensions));
	emxArray->allocatedSize = 0;
	emxArray->canFreeData = true;
	for (i = 0; i < numDimensions; i++) {
		emxArray->size[i] = 0;
	}
}

//
// File trailer for devita_emxutil.cpp
//
// [EOF]
//
// Function Definitions

//
// Arguments    : int numDimensions
//                int *size
// Return Type  : emxArray_real_T *
//
emxArray_real_T *emxCreateND_real_T(int numDimensions, int *size)
{
	emxArray_real_T *emx;
	int numEl;
	int i;
	emxInit_real_T(&emx, numDimensions);
	numEl = 1;
	for (i = 0; i < numDimensions; i++) {
		numEl *= size[i];
		emx->size[i] = size[i];
	}

	emx->data = (double *)calloc((unsigned int)numEl, sizeof(double));
	emx->numDimensions = numDimensions;
	emx->allocatedSize = numEl;
	return emx;
}

//
// Arguments    : double *data
//                int numDimensions
//                int *size
// Return Type  : emxArray_real_T *
//
emxArray_real_T *emxCreateWrapperND_real_T(double *data, int numDimensions, int *
	size)
{
	emxArray_real_T *emx;
	int numEl;
	int i;
	emxInit_real_T(&emx, numDimensions);
	numEl = 1;
	for (i = 0; i < numDimensions; i++) {
		numEl *= size[i];
		emx->size[i] = size[i];
	}

	emx->data = data;
	emx->numDimensions = numDimensions;
	emx->allocatedSize = numEl;
	emx->canFreeData = false;
	return emx;
}

//
// Arguments    : double *data
//                int rows
//                int cols
// Return Type  : emxArray_real_T *
//
emxArray_real_T *emxCreateWrapper_real_T(double *data, int rows, int cols)
{
	emxArray_real_T *emx;
	int size[2];
	int numEl;
	int i;
	size[0] = rows;
	size[1] = cols;
	emxInit_real_T(&emx, 2);
	numEl = 1;
	for (i = 0; i < 2; i++) {
		numEl *= size[i];
		emx->size[i] = size[i];
	}

	emx->data = data;
	emx->numDimensions = 2;
	emx->allocatedSize = numEl;
	emx->canFreeData = false;
	return emx;
}

//
// Arguments    : int rows
//                int cols
// Return Type  : emxArray_real_T *
//
emxArray_real_T *emxCreate_real_T(int rows, int cols)
{
	emxArray_real_T *emx;
	int size[2];
	int numEl;
	int i;
	size[0] = rows;
	size[1] = cols;
	emxInit_real_T(&emx, 2);
	numEl = 1;
	for (i = 0; i < 2; i++) {
		numEl *= size[i];
		emx->size[i] = size[i];
	}

	emx->data = (double *)calloc((unsigned int)numEl, sizeof(double));
	emx->numDimensions = 2;
	emx->allocatedSize = numEl;
	return emx;
}

//
// Arguments    : emxArray_real_T *emxArray
// Return Type  : void
//
void emxDestroyArray_real_T(emxArray_real_T *emxArray)
{
	emxFree_real_T(&emxArray);
}

//
// Arguments    : emxArray_real_T **pEmxArray
//                int numDimensions
// Return Type  : void
//
void emxInitArray_real_T(emxArray_real_T **pEmxArray, int numDimensions)
{
	emxInit_real_T(pEmxArray, numDimensions);
}

//
// File trailer for devita_emxAPI.cpp
//
// [EOF]
//

// Function Declarations
static void __anon_fcn(double alpha, double beta, double eta, double i, double
	epsilondot, double k, const emxArray_real_T *x, emxArray_real_T *varargout_1);
static void b_exp(emxArray_real_T *x);
static boolean_T b_isfinite(double x);
static boolean_T mesh_has_collapsed(const emxArray_real_T *x);
static double midpArea(const double f_environment[6], double b);
static void power(const emxArray_real_T *a, double b, emxArray_real_T *y);
static double quadgk(const double fun_environment[6], double b);
static double rt_powd_snf(double u0, double u1);
static void split(double x[650], int nx, int *nxnew, double *pathlen);
static void transfun(signed char problem_type, const double f_environment[6],
	const emxArray_real_T *t, double B, boolean_T firstcall,
	emxArray_real_T *y, boolean_T *tooclose);

// Function Definitions

//
// Arguments    : double alpha
//                double beta
//                double eta
//                double i
//                double epsilondot
//                double k
//                const emxArray_real_T *x
//                emxArray_real_T *varargout_1
// Return Type  : void
//
static void __anon_fcn(double alpha, double beta, double eta, double i, double
	epsilondot, double k, const emxArray_real_T *x, emxArray_real_T *varargout_1)
{
	int b_varargout_1;
	double b_alpha;
	int c_varargout_1;
	emxArray_real_T *b_x;
	emxArray_real_T *r0;

	// 'st:2' p=alpha.*beta.^(-alpha).*x.^(alpha-1).*exp(-(x./beta).^alpha).*(k.*(epsilon-x)+eta.*epsilondot); 
	power(x, alpha - 1.0, varargout_1);
	b_varargout_1 = varargout_1->size[0] * varargout_1->size[1];
	varargout_1->size[0] = 1;
	emxEnsureCapacity_real_T1(varargout_1, b_varargout_1);
	b_alpha = alpha * rt_powd_snf(beta, -alpha);
	c_varargout_1 = varargout_1->size[0];
	b_varargout_1 = varargout_1->size[1];
	c_varargout_1 *= b_varargout_1;
	for (b_varargout_1 = 0; b_varargout_1 < c_varargout_1; b_varargout_1++) {
		varargout_1->data[b_varargout_1] *= b_alpha;
	}

	emxInit_real_T1(&b_x, 2);
	b_varargout_1 = b_x->size[0] * b_x->size[1];
	b_x->size[0] = 1;
	b_x->size[1] = x->size[1];
	emxEnsureCapacity_real_T1(b_x, b_varargout_1);
	c_varargout_1 = x->size[0] * x->size[1];
	for (b_varargout_1 = 0; b_varargout_1 < c_varargout_1; b_varargout_1++) {
		b_x->data[b_varargout_1] = x->data[b_varargout_1] / beta;
	}

	emxInit_real_T1(&r0, 2);
	power(b_x, alpha, r0);
	b_varargout_1 = r0->size[0] * r0->size[1];
	r0->size[0] = 1;
	emxEnsureCapacity_real_T1(r0, b_varargout_1);
	b_varargout_1 = r0->size[0];
	c_varargout_1 = r0->size[1];
	c_varargout_1 *= b_varargout_1;
	emxFree_real_T(&b_x);
	for (b_varargout_1 = 0; b_varargout_1 < c_varargout_1; b_varargout_1++) {
		r0->data[b_varargout_1] = -r0->data[b_varargout_1];
	}

	b_exp(r0);
	b_varargout_1 = varargout_1->size[0] * varargout_1->size[1];
	varargout_1->size[0] = 1;
	emxEnsureCapacity_real_T1(varargout_1, b_varargout_1);
	c_varargout_1 = varargout_1->size[0];
	b_varargout_1 = varargout_1->size[1];
	b_alpha = eta * epsilondot;
	c_varargout_1 *= b_varargout_1;
	for (b_varargout_1 = 0; b_varargout_1 < c_varargout_1; b_varargout_1++) {
		varargout_1->data[b_varargout_1] = varargout_1->data[b_varargout_1] *
			r0->data[b_varargout_1] * (k * (i - x->data[b_varargout_1]) + b_alpha);
	}

	emxFree_real_T(&r0);
}

//
// Arguments    : emxArray_real_T *x
// Return Type  : void
//
static void b_exp(emxArray_real_T *x)
{
	int nx;
	int k;
	nx = x->size[1];
	for (k = 0; k + 1 <= nx; k++) {
		x->data[k] = exp(x->data[k]);
	}
}

//
// Arguments    : double x
// Return Type  : boolean_T
//
static boolean_T b_isfinite(double x)
{
	return (!rtIsInf(x)) && (!rtIsNaN(x));
}

//
// Arguments    : const emxArray_real_T *x
// Return Type  : boolean_T
//
static boolean_T mesh_has_collapsed(const emxArray_real_T *x)
{
	boolean_T p;
	double absxk;
	int k;
	boolean_T exitg1;
	double absxkm1;
	double b_absxkm1;
	absxk = fabs(x->data[0]);
	p = false;
	k = 0;
	exitg1 = false;
	while ((!exitg1) && (k <= x->size[1] - 2)) {
		absxkm1 = absxk;
		absxk = fabs(x->data[k + 1]);
		if ((absxkm1 > absxk) || rtIsNaN(absxk)) {
			b_absxkm1 = absxkm1;
		}
		else {
			b_absxkm1 = absxk;
		}

		if (fabs(x->data[k + 1] - x->data[k]) <= 2.2204460492503131E-14 * b_absxkm1)
		{
			p = true;
			exitg1 = true;
		}
		else {
			k++;
		}
	}

	return p;
}

//
// Arguments    : const double f_environment[6]
//                double b
// Return Type  : double
//
static double midpArea(const double f_environment[6], double b)
{
	double fx;
	fx = st(b / 2.0, f_environment[0], f_environment[1], f_environment[2],
		f_environment[3], f_environment[4], f_environment[5]);
	return b * fx;
}

//
// Arguments    : const emxArray_real_T *a
//                double b
//                emxArray_real_T *y
// Return Type  : void
//
static void power(const emxArray_real_T *a, double b, emxArray_real_T *y)
{
	short unnamed_idx_1;
	int k;
	unnamed_idx_1 = (short)a->size[1];
	k = y->size[0] * y->size[1];
	y->size[0] = 1;
	y->size[1] = (short)a->size[1];
	emxEnsureCapacity_real_T1(y, k);
	for (k = 0; k + 1 <= unnamed_idx_1; k++) {
		y->data[k] = rt_powd_snf(a->data[k], b);
	}
}

//
// Arguments    : const double fun_environment[6]
//                double b
// Return Type  : double
//
static double quadgk(const double fun_environment[6], double b)
{
	double q;
	double d0;
	double d1;
	double interval[650];
	int nt;
	signed char problem_type;
	double pathlen;
	int nsubs;
	int k;
	double q_ok;
	double subs[1298];
	double err_ok;
	boolean_T first_iteration;
	emxArray_real_T *x;
	emxArray_real_T *fx;
	int exitg1;
	boolean_T tooclose;
	double qsubs;
	double errsub1norm;
	int j;
	static const double NODES[15] = { -0.99145537112081261, -0.94910791234275849,
	  -0.8648644233597691, -0.74153118559939435, -0.58608723546769115,
	  -0.40584515137739718, -0.20778495500789851, 0.0, 0.20778495500789851,
	  0.40584515137739718, 0.58608723546769115, 0.74153118559939435,
	  0.8648644233597691, 0.94910791234275849, 0.99145537112081261 };

	double qsub[649];
	double errsub[649];
	double tol;
	static const double dv0[15] = { 0.022935322010529221, 0.063092092629978544,
	  0.1047900103222502, 0.14065325971552589, 0.16900472663926791,
	  0.19035057806478539, 0.20443294007529891, 0.20948214108472779,
	  0.20443294007529891, 0.19035057806478539, 0.16900472663926791,
	  0.14065325971552589, 0.1047900103222502, 0.063092092629978544,
	  0.022935322010529221 };

	static const double dv1[15] = { 0.022935322010529221, -0.066392873538891159,
	  0.1047900103222502, -0.13905213177375081, 0.16900472663926791,
	  -0.19147947244033353, 0.20443294007529891, -0.20847704258874161,
	  0.20443294007529891, -0.19147947244033353, 0.16900472663926791,
	  -0.13905213177375081, 0.1047900103222502, -0.066392873538891159,
	  0.022935322010529221 };

	double abserrsubk;
	if (0.0 != b) {
		d0 = 4.94065645841247E-324;
	}
	else {
		d0 = 0.0;
	}

	st(d0, fun_environment[0], fun_environment[1], fun_environment[2],
		fun_environment[3], fun_environment[4], fun_environment[5]);
	if (0.0 != b) {
		d1 = 4.94065645841247E-324;
	}
	else {
		d1 = 0.0;
	}

	st(d1, fun_environment[0], fun_environment[1], fun_environment[2],
		fun_environment[3], fun_environment[4], fun_environment[5]);
	interval[0] = 0.0;
	nt = 2;
	interval[1] = b;
	if (0.0 >= b) {
		problem_type = 0;
	}
	else {
		interval[0] = -1.0;
		interval[1] = 1.0;
		problem_type = 1;
	}

	memset(&interval[2], 0, 648U * sizeof(double));
	q = 0.0;
	if (problem_type == 0) {
		pathlen = 0.0;
	}
	else {
		split(interval, 2, &nt, &pathlen);
		if (!(pathlen > 0.0)) {
			problem_type = 0;
		}
	}

	if (problem_type == 0) {
		q = midpArea(fun_environment, b);
	}
	else {
		nsubs = nt - 1;
		for (k = 0; k + 1 < nt; k++) {
			subs[k << 1] = interval[k];
			subs[1 + (k << 1)] = interval[k + 1];
		}

		q_ok = 0.0;
		err_ok = 0.0;
		first_iteration = true;
		emxInit_real_T1(&x, 2);
		emxInit_real_T1(&fx, 2);
		do {
			exitg1 = 0;
			nt = x->size[0] * x->size[1];
			x->size[0] = 1;
			x->size[1] = 15 * nsubs;
			emxEnsureCapacity_real_T1(x, nt);
			nt = -1;
			for (k = 0; k + 1 <= nsubs; k++) {
				qsubs = (subs[k << 1] + subs[1 + (k << 1)]) / 2.0;
				errsub1norm = (subs[1 + (k << 1)] - subs[k << 1]) / 2.0;
				for (j = 0; j < 15; j++) {
					nt++;
					x->data[nt] = NODES[j] * errsub1norm + qsubs;
				}
			}

			transfun(1, fun_environment, x, b, first_iteration, fx, &tooclose);
			if (tooclose) {
				exitg1 = 1;
			}
			else {
				qsubs = 0.0;
				nt = -1;
				for (k = 0; k + 1 <= nsubs; k++) {
					qsub[k] = 0.0;
					errsub[k] = 0.0;
					for (j = 0; j < 15; j++) {
						nt++;
						qsub[k] += dv0[j] * fx->data[nt];
						errsub[k] += dv1[j] * fx->data[nt];
					}

					errsub1norm = (subs[1 + (k << 1)] - subs[k << 1]) / 2.0;
					qsub[k] *= errsub1norm;
					qsubs += qsub[k];
					errsub[k] *= errsub1norm;
				}

				q = qsubs + q_ok;
				qsubs = 1.0E-6 * fabs(q);
				if ((1.0E-10 > qsubs) || rtIsNaN(qsubs)) {
					tol = 1.0E-10;
				}
				else {
					tol = qsubs;
				}

				qsubs = 2.0 * tol / pathlen;
				errsub1norm = 0.0;
				nt = 0;
				for (k = 0; k + 1 <= nsubs; k++) {
					abserrsubk = fabs(errsub[k]);
					if (abserrsubk <= qsubs * ((subs[1 + (k << 1)] - subs[k << 1]) / 2.0))
					{
						err_ok += errsub[k];
						q_ok += qsub[k];
					}
					else {
						errsub1norm += abserrsubk;
						nt++;
						subs[(nt - 1) << 1] = subs[k << 1];
						subs[1 + ((nt - 1) << 1)] = subs[1 + (k << 1)];
					}
				}

				qsubs = fabs(err_ok) + errsub1norm;
				if (b_isfinite(q) && b_isfinite(qsubs)) {
					first_iteration = true;
				}
				else {
					first_iteration = false;
				}

				if ((!first_iteration) || (nt == 0) || (qsubs <= tol)) {
					exitg1 = 1;
				}
				else {
					nsubs = nt << 1;
					if (nsubs > 650) {
						exitg1 = 1;
					}
					else {
						while (nt > 0) {
							subs[1 + (((nt << 1) - 1) << 1)] = subs[1 + ((nt - 1) << 1)];
							subs[((nt << 1) - 1) << 1] = (subs[(nt - 1) << 1] + subs[1 + ((nt
								- 1) << 1)]) / 2.0;
							subs[1 + (((nt << 1) - 2) << 1)] = subs[((nt << 1) - 1) << 1];
							subs[((nt << 1) - 2) << 1] = subs[(nt - 1) << 1];
							nt--;
						}

						first_iteration = false;
					}
				}
			}
		} while (exitg1 == 0);

		emxFree_real_T(&fx);
		emxFree_real_T(&x);
	}

	return q;
}

//
// Arguments    : double u0
//                double u1
// Return Type  : double
//
static double rt_powd_snf(double u0, double u1)
{
	double y;
	double d2;
	double d3;
	if (rtIsNaN(u0) || rtIsNaN(u1)) {
		y = rtNaN;
	}
	else {
		d2 = fabs(u0);
		d3 = fabs(u1);
		if (rtIsInf(u1)) {
			if (d2 == 1.0) {
				y = 1.0;
			}
			else if (d2 > 1.0) {
				if (u1 > 0.0) {
					y = rtInf;
				}
				else {
					y = 0.0;
				}
			}
			else if (u1 > 0.0) {
				y = 0.0;
			}
			else {
				y = rtInf;
			}
		}
		else if (d3 == 0.0) {
			y = 1.0;
		}
		else if (d3 == 1.0) {
			if (u1 > 0.0) {
				y = u0;
			}
			else {
				y = 1.0 / u0;
			}
		}
		else if (u1 == 2.0) {
			y = u0 * u0;
		}
		else if ((u1 == 0.5) && (u0 >= 0.0)) {
			y = sqrt(u0);
		}
		else if ((u0 < 0.0) && (u1 > floor(u1))) {
			y = rtNaN;
		}
		else {
			y = pow(u0, u1);
		}
	}

	return y;
}

//
// Arguments    : double x[650]
//                int nx
//                int *nxnew
//                double *pathlen
// Return Type  : void
//
static void split(double x[650], int nx, int *nxnew, double *pathlen)
{
	int n_idx_0;
	int ridx;
	double delta;
	int j;
	*pathlen = x[1] - x[0];
	if (*pathlen > 0.0) {
		n_idx_0 = (int)(ceil(fabs(x[1] - x[0]) * (10.0 / *pathlen)) - 1.0);
		*nxnew = 2 + n_idx_0;
		if (2 + n_idx_0 > 2) {
			ridx = n_idx_0;
			x[n_idx_0 + 1] = x[1];
			delta = (x[1] - x[0]) / (double)(n_idx_0 + 1);
			for (j = n_idx_0; j > 0; j--) {
				x[ridx] = x[0] + (double)j * delta;
				ridx--;
			}
		}

		nx = 2 + n_idx_0;
	}
	else {
		*nxnew = 2;
	}

	n_idx_0 = 0;
	for (ridx = 1; ridx + 1 <= nx; ridx++) {
		if (fabs(x[ridx] - x[n_idx_0]) > 0.0) {
			n_idx_0++;
			x[n_idx_0] = x[ridx];
		}
		else {
			(*nxnew)--;
		}
	}

	if (*nxnew < 2) {
		x[1] = x[nx - 1];
		*nxnew = 2;
	}
}

//
// Arguments    : signed char problem_type
//                const double f_environment[6]
//                const emxArray_real_T *t
//                double B
//                boolean_T firstcall
//                emxArray_real_T *y
//                boolean_T *tooclose
// Return Type  : void
//
static void transfun(signed char problem_type, const double f_environment[6],
	const emxArray_real_T *t, double B, boolean_T firstcall,
	emxArray_real_T *y, boolean_T *tooclose)
{
	emxArray_real_T *x;
	emxArray_real_T *xt;
	int nt;
	int i2;
	double BmAd4;
	int k;
	double BpAd2;
	double BmAtp75;
	double tk2;
	boolean_T b_tooclose;
	short unnamed_idx_1;
	emxInit_real_T1(&x, 2);
	emxInit_real_T1(&xt, 2);
	nt = t->size[1] - 1;
	i2 = x->size[0] * x->size[1];
	x->size[0] = t->size[0];
	x->size[1] = t->size[1];
	emxEnsureCapacity_real_T1(x, i2);
	i2 = xt->size[0] * xt->size[1];
	xt->size[0] = t->size[0];
	xt->size[1] = t->size[1];
	emxEnsureCapacity_real_T1(xt, i2);
	if (problem_type == 1) {
		BmAd4 = 0.25 * B;
		BpAd2 = 0.5 * B;
		BmAtp75 = 0.75 * B;
		for (k = 0; k < t->size[1]; k++) {
			tk2 = t->data[k] * t->data[k];
			x->data[k] = BmAd4 * t->data[k] * (3.0 - tk2) + BpAd2;
			xt->data[k] = BmAtp75 * (1.0 - tk2);
		}
	}
	else if (problem_type == 2) {
		for (k = 0; k <= nt; k++) {
			tk2 = t->data[k] / (1.0 - t->data[k]);
			x->data[k] = tk2 * tk2;
			xt->data[k] = 2.0 * tk2 / ((1.0 - t->data[k]) * (1.0 - t->data[k]));
		}
	}
	else if (problem_type == 3) {
		for (k = 0; k <= nt; k++) {
			tk2 = t->data[k] / (1.0 + t->data[k]);
			x->data[k] = B - tk2 * tk2;
			xt->data[k] = -2.0 * tk2 / ((1.0 + t->data[k]) * (1.0 + t->data[k]));
		}
	}
	else {
		for (k = 0; k <= nt; k++) {
			tk2 = t->data[k] * t->data[k];
			x->data[k] = t->data[k] / (1.0 - tk2);
			xt->data[k] = (1.0 + tk2) / ((1.0 - tk2) * (1.0 - tk2));
		}
	}

	if ((!firstcall) && mesh_has_collapsed(x)) {
		b_tooclose = true;
	}
	else {
		b_tooclose = false;
	}

	if (b_tooclose) {
		unnamed_idx_1 = (short)t->size[1];
		i2 = y->size[0] * y->size[1];
		y->size[0] = 1;
		y->size[1] = unnamed_idx_1;
		emxEnsureCapacity_real_T1(y, i2);
		k = unnamed_idx_1;
		for (i2 = 0; i2 < k; i2++) {
			y->data[i2] = 0.0;
		}
	}
	else {
		__anon_fcn(f_environment[0], f_environment[1], f_environment[2],
			f_environment[3], f_environment[4], f_environment[5], x, y);
		i2 = y->size[0] * y->size[1];
		y->size[0] = 1;
		emxEnsureCapacity_real_T1(y, i2);
		k = y->size[0];
		nt = y->size[1];
		k *= nt;
		for (i2 = 0; i2 < k; i2++) {
			y->data[i2] *= xt->data[i2];
		}
	}

	emxFree_real_T(&xt);
	emxFree_real_T(&x);
	*tooclose = b_tooclose;
}

//
// function theor_stress=devita(alpha,beta,eta,epsilon,epsilondot,k)
// Arguments    : double alpha
//                double beta
//                double eta
//                double epsilon
//                double epsilondot
//                double k
//                emxArray_real_T *theor_stress
// Return Type  : void
//
void devita(double alpha, double beta, double eta, double epsilon, double
	epsilondot, double k, emxArray_real_T *theor_stress)
{
	emxArray_real_T *pxx;
	int i0;
	double environment[6];
	int i;
	double p;
	int i1;
	double this_environment[6];
	int xs;
	emxInit_real_T(&pxx, 1);

	// 'devita:2' pxx= 0;
	i0 = pxx->size[0];
	pxx->size[0] = 1;
	emxEnsureCapacity_real_T(pxx, i0);
	pxx->data[0] = 0.0;

	// 'devita:2' theor_stress = 0;
	i0 = theor_stress->size[0];
	theor_stress->size[0] = 1;
	emxEnsureCapacity_real_T(theor_stress, i0);
	theor_stress->data[0] = 0.0;

	// 'devita:3' for i=0:0.01:epsilon
	i0 = (int)((epsilon + 0.01) / 0.01);
	if (0 <= i0 - 1) {
		environment[0] = alpha;
		environment[1] = beta;
		environment[2] = eta;
		environment[4] = epsilondot;
		environment[5] = k;
	}

	for (i = 0; i < i0; i++) {
		p = (double)i * 0.01;

		// 'devita:4' p=quadgk(@(x)st(x,alpha,beta,eta,i,epsilondot,k),0,i);
		environment[3] = p;
		for (i1 = 0; i1 < 6; i1++) {
			this_environment[i1] = environment[i1];
		}

		p = quadgk(this_environment, p);

		// 'devita:5' pxx=[pxx;p];
		xs = pxx->size[0];
		i1 = pxx->size[0];
		pxx->size[0] = xs + 1;
		emxEnsureCapacity_real_T(pxx, i1);
		pxx->data[xs] = p;

		// 'devita:6' theor_stress= pxx;
		i1 = theor_stress->size[0];
		theor_stress->size[0] = pxx->size[0];
		emxEnsureCapacity_real_T(theor_stress, i1);
		xs = pxx->size[0];
		for (i1 = 0; i1 < xs; i1++) {
			theor_stress->data[i1] = pxx->data[i1];
		}
	}

	emxFree_real_T(&pxx);
}

//
// Arguments    : void
// Return Type  : void
//
void devita_initialize()
{
	rt_InitInfAndNaN(8U);
}

//
// Arguments    : void
// Return Type  : void
//
void devita_terminate()
{
	// (no terminate code required)
}

//
// function p=st(x,alpha,beta,eta,epsilon,epsilondot,k)
// Arguments    : double x
//                double alpha
//                double beta
//                double eta
//                double epsilon
//                double epsilondot
//                double k
// Return Type  : double
//
double st(double x, double alpha, double beta, double eta, double epsilon,
	double epsilondot, double k)
{
	// 'st:2' p=alpha.*beta.^(-alpha).*x.^(alpha-1).*exp(-(x./beta).^alpha).*(k.*(epsilon-x)+eta.*epsilondot); 
	return alpha * rt_powd_snf(beta, -alpha) * rt_powd_snf(x, alpha - 1.0) * exp
	(-rt_powd_snf(x / beta, alpha)) * (k * (epsilon - x) + eta * epsilondot);
}

//
// File trailer for devita.cpp
//
// [EOF]
//






// Function Declarations

static double argInit_real_T();
static void main_devita();
static void main_st();
static double argInit_real_T()
{
	return 0.0;
}
double A[100],B[100];
int n;

using namespace std;
using namespace OpenSim;
using SimTK::Vec3;

NonLinearLigament::NonLinearLigament()
{
	constructProperties();
}

void NonLinearLigament::constructProperties()
{
	setAuthors("Arnab Sikidar");
	constructProperty_GeometryPath(GeometryPath());
	constructProperty_resting_length(0.0);
	constructProperty_pcsa_force(0.0);
	constructProperty_eta_a(NULL);
	constructProperty_alphaa_a(NULL);
	constructProperty_beta_a(NULL);
	constructProperty_ka_a(NULL);
	constructProperty_areaa_a(NULL);
	constructProperty_max_strain(NULL);
	//constructProperty_strain_rate(NULL);
	/*int forceLengthCurvePoints = 13;
	double forceLengthCurveX[] = { -5.00000000,  0.99800000,  0.99900000,  1.00000000,  1.10000000,  1.20000000,  1.30000000,  1.40000000,  1.50000000,  1.60000000,  1.60100000,  1.60200000,  5.00000000 };
	double forceLengthCurveY[] = { 0.00000000,  0.00000000,  0.00000000,  0.00000000,  0.03500000,  0.12000000,  0.26000000,  0.55000000,  1.17000000,  2.00000000,  2.00000000,  2.00000000,  2.00000000 };
	SimmSpline forceLengthCurve(forceLengthCurvePoints, forceLengthCurveX, forceLengthCurveY);

	constructProperty_force_length_curve(forceLengthCurve);
	*/
}


//Global Variables
double x;
double eta, alpha, beta, k, epsilondot, epsilon;
double epsilon_Array[100];
static const Vec3 DefaultLigamentColor(.9, .9, .9); // mostly white 

void NonLinearLigament::connectToModel(Model& aModel)
{
	GeometryPath& path = upd_GeometryPath();
	const double& restingLength = get_resting_length();

	path.setDefaultColor(DefaultLigamentColor);

	// Specify underlying ModelComponents prior to calling 
	// Super::connectToModel() to automatically propagate connectToModel()
	// to subcomponents. Subsequent addToSystem() will also be automatically
	// propagated to subcomponents.
	// TODO: this is awkward; subcomponent API needs to be revisited (sherm)
	includeAsSubComponent(&path);

	//TODO: can't call this at start of override; this is an API bug.
	Super::connectToModel(aModel);

	// _model will be NULL when objects are being registered.
	if (_model == NULL)
		return;

	// Resting length must be greater than 0.0.
	assert(restingLength > 0.0);

	path.setOwner(this);
}

void NonLinearLigament::realizeDynamics(const SimTK::State& state) const {
	Super::realizeDynamics(state); // Mandatory first line

	if (!isDisabled(state)) {
		const SimTK::Vec3 color = computePathColor(state);
		if (!color.isNaN())
			getGeometryPath().setColor(state, color);
	}
}

SimTK::Vec3 NonLinearLigament::computePathColor(const SimTK::State& state) const {
	return SimTK::Vec3(SimTK::NaN);
}

void NonLinearLigament::addToSystem(SimTK::MultibodySystem& system) const
{
	Super::addToSystem(system);
	// Cache the computed tension and strain of the ligament
	addCacheVariable<double>("tension", 0.0, SimTK::Stage::Velocity);
	addCacheVariable<double>("strain", 0.0, SimTK::Stage::Velocity);
}

double NonLinearLigament::getLength(const SimTK::State& s) const
{
	return getGeometryPath().getLength(s);
}

bool NonLinearLigament::setRestingLength(double aRestingLength)
{
	set_resting_length(aRestingLength);
	return true;
}

bool NonLinearLigament::setMaxIsometricForce(double aMaxIsometricForce)
{
	set_pcsa_force(aMaxIsometricForce);
	return true;
}


/*bool NonLinearLigament::setForceLengthCurve()
{
	set_force_length_curve(DevitaModel());
	return true;
}*/

void NonLinearLigament::preScale(const SimTK::State& s, const ScaleSet& aScaleSet)
{
	updGeometryPath().preScale(s, aScaleSet);
}

void NonLinearLigament::scale(const SimTK::State& s, const ScaleSet& aScaleSet)
{
	updGeometryPath().scale(s, aScaleSet);
}

void NonLinearLigament::postScale(const SimTK::State& s, const ScaleSet& aScaleSet)
{
	GeometryPath& path = updGeometryPath();
	double&       restingLength = upd_resting_length();

	path.postScale(s, aScaleSet);

	if (path.getPreScaleLength(s) > 0.0)
	{
		double scaleFactor = path.getLength(s) / path.getPreScaleLength(s);

		// Scale resting length by the same amount as the change in
		// total ligament length (in the current body position).
		restingLength *= scaleFactor;

		path.setPreScaleLength(s, 0.0);
	}
}

const double& NonLinearLigament::getTension(const SimTK::State& s) const
{
	return getCacheVariable<double>(s, "tension");
}

double NonLinearLigament::computeMomentArm(const SimTK::State& s, Coordinate& aCoord) const
{
	return getGeometryPath().computeMomentArm(s, aCoord);
}




const VisibleObject* NonLinearLigament::getDisplayer() const
{
	return getGeometryPath().getDisplayer();
}

//_____________________________________________________________________________
/**
* Update the visible object used to represent the Ligament.
*/
void NonLinearLigament::updateDisplayer(const SimTK::State& s) const
{
	getGeometryPath().updateDisplayer(s);
}

SimmSpline Active1,Active2;

static void main_devita()
{

	emxArray_real_T *theor_stress;
	emxInitArray_real_T(&theor_stress, 1);

	// Initialize function 'devita' input arguments.
	// Call the entry-point 'devita'.
	devita(alpha, beta, eta, epsilon, epsilondot, k, theor_stress);
	for (int i = 0; i< *theor_stress->size; i++)
	{
		A[i] = theor_stress->data[i];
	}
	n = *theor_stress->size;
	emxDestroyArray_real_T(theor_stress);
}

static void main_st()
{
	double p;

	// Initialize function 'st' input arguments.
	// Call the entry-point 'st'.
	p = st(argInit_real_T(), argInit_real_T(), argInit_real_T(), argInit_real_T(),
		argInit_real_T(), argInit_real_T(), argInit_real_T());
}




bool NonLinearLigament::setEtaA(double eta)
{
	set_eta_a(eta);
	return true;
}

void NonLinearLigament::setAlphaA(double alpha)
{
	set_alphaa_a(alpha);
}

bool NonLinearLigament::setBetaA(double beta)
{
	set_beta_a(beta);
	return true;
}

bool NonLinearLigament::setKaA(double k)
{
	set_ka_a(k);
	return true;
}

bool NonLinearLigament::setAreaA(double a)
{	
	
	set_areaa_a(a);
	return true;
}

bool NonLinearLigament::setMaxStrain(double a)
{

	set_max_strain(a);
	return true;
}

double lemda[100];

/*void NonLinearLigament::setStrainRate(double astrain_rate)
{
	set_strain_rate(astrain_rate);
	//std::cout << "*";
	//DevitaModel();
}
*/
SimmSpline NonLinearLigament::DevitaModel(const SimTK::State& s) const
{
	//double p = 0;
	eta = getEtaA();
	alpha = getAlphaA();
	beta = getBetaA();
	k = getKaA();
	epsilon = getMaxStrain();
	double length=getRestingLength();
	double pcsa_force = getMaxIsometricForce();
	double pcsa = getAreaA();
	epsilondot = getLengtheningSpeed(s)/length;
	devita_initialize();

	// Invoke the entry-point functions.
	// You can call entry-point functions multiple times.
	main_devita();
	main_st();

	// Terminate the application.
	// You do not need to do this more than one time.
	devita_terminate();
	double a,  ep[100];
	double Force_Multiplier[100];
	a = 0;
	//fstream myfile;
	for (int i = 0; i < n - 1; i++)
	{
		ep[i] = a;
		a += 0.01;
		lemda[i] = (100 * length * (exp(ep[i]) - 1));
	}
	
	//myfile.open("file.txt");
	for (int i = 0; i < n-1; i++)
	{
		Force_Multiplier[i] = A[i] * pcsa / pcsa_force;
		lemda[i] = (lemda[i]+length*100)/(100*length);
		//lemda[i] = lemda[i] / min;
		//std::cout << lemda[i] << "\t" << Force_Multiplier[i] << "\n";
		//myfile << lemda[i]  << "\t" << Force_Multiplier[i] << "\n";
	}
	//myfile.close();
	//exit(0);
	SimmSpline activeForceLengthCurve(n-1, lemda, Force_Multiplier);
	Active1 = activeForceLengthCurve;
	return Active1;
}

double NonLinearLigament::getLengtheningSpeed(const SimTK::State& s) const
{
	return get_GeometryPath().getLengtheningSpeed(s);
}


void NonLinearLigament::computeForce(const SimTK::State& s,
	SimTK::Vector_<SimTK::SpatialVec>& bodyForces,
	SimTK::Vector& generalizedForces) const
{
	const GeometryPath& path = getGeometryPath();
	const double& restingLength = get_resting_length();
	double force = 0;

	if (path.getLength(s) <= restingLength) {
		setCacheVariable<double>(s, "tension", force);
		return;
	}
	eta = getEtaA();
	alpha = getAlphaA();
	beta = getBetaA();
	k = getKaA();
	//epsilondot = getStrainRate();
	double pcsa_force = getMaxIsometricForce();
	double pcsa = getAreaA();
	Active2 = DevitaModel(s);
	// evaluate normalized tendon force length curve
	
	force = Active2.calcValue(
		SimTK::Vector(1, path.getLength(s) / restingLength))* pcsa_force;

	setCacheVariable<double>(s, "tension", force);
	OpenSim::Array<PointForceDirection*> PFDs;
	path.getPointForceDirections(s, &PFDs);

	for (int i = 0; i < PFDs.getSize(); i++) {
		applyForceToPoint(s, PFDs[i]->body(), PFDs[i]->point(),
			force*PFDs[i]->direction(), bodyForces);
	}
	for (int i = 0; i < PFDs.getSize(); i++)
		delete PFDs[i];
}

/*
double NonLinearLigament::computePotentialEnergy(const SimTK::State& s)
const {
	const GeometryPath& path = getGeometryPath();
	const double& restingLength = get_resting_length();

	double force = 0;

	if (path.getLength(s) <= restingLength) {
		setCacheVariable<double>(s, "tension", force);
		return;
	}
	eta = getEtaA();
	alpha = getAlphaA();
	beta = getBetaA();
	k = getKaA();
	epsilondot = getStrainRate();
	double pcsa_force = getMaxIsometricForce();
	double pcsa = getAreaA();

	// evaluate normalized tendon force length curve

	force = ForceLengthCurve().calcValue(SimTK::Vector(1, (getLength(s)) / restingLength))* pcsa_force * pcsa;
	
	return force * getLength(s);
}
*/