#ifndef OPENSIM_NON_LINEAR_LIGAMENT_H_
#define OPENSIM_NON_LINEAR_LIGAMENT_H_

// INCLUDE
#include <OpenSim/OpenSim.h>
#include <OpenSim/Common/ScaleSet.h>
#include <OpenSim/Simulation/Model/Model.h>
#include <OpenSim/Simulation/Model/Force.h>
#include <math.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#if defined(_MSC_VER) && (_MSC_VER <= 1200)
#include <float.h>
#endif

//rt_nonfinite.h
//rtwtypes.h
#ifndef __TMWTYPES__					
#define __TMWTYPES__

/*=======================================================================*
 * Target hardware information
 *   Device type: Generic->MATLAB Host Computer
 *   Number of bits:     char:   8    short:   16    int:  32
 *                       long:  32    long long:  64
 *                       native word size:  64
 *   Byte ordering: LittleEndian
 *   Signed integer division rounds to: Zero
 *   Shift right on a signed integer as arithmetic shift: on
 *=======================================================================*/

 /*=======================================================================*
  * Fixed width word size data types:                                     *
  *   int8_T, int16_T, int32_T     - signed 8, 16, or 32 bit integers     *
  *   uint8_T, uint16_T, uint32_T  - unsigned 8, 16, or 32 bit integers   *
  *   real32_T, real64_T           - 32 and 64 bit floating point numbers *
  *=======================================================================*/
typedef signed char int8_T;
typedef unsigned char uint8_T;
typedef short int16_T;
typedef unsigned short uint16_T;
typedef int int32_T;
typedef unsigned int uint32_T;
typedef long long int64_T;
typedef unsigned long long uint64_T;
typedef float real32_T;
typedef double real64_T;

/*===========================================================================*
 * Generic type definitions: real_T, time_T, boolean_T, int_T, uint_T,       *
 *                           ulong_T, ulonglong_T, char_T and byte_T.        *
 *===========================================================================*/
typedef double real_T;
typedef double time_T;
typedef unsigned char boolean_T;
typedef int int_T;
typedef unsigned int uint_T;
typedef unsigned long ulong_T;
typedef unsigned long long ulonglong_T;
typedef char char_T;
typedef char_T byte_T;

/*===========================================================================*
 * Complex number type definitions                                           *
 *===========================================================================*/
#define CREAL_T

typedef struct {
	real32_T re;
	real32_T im;
} creal32_T;

typedef struct {
	real64_T re;
	real64_T im;
} creal64_T;

typedef struct {
	real_T re;
	real_T im;
} creal_T;

typedef struct {
	int8_T re;
	int8_T im;
} cint8_T;

typedef struct {
	uint8_T re;
	uint8_T im;
} cuint8_T;

typedef struct {
	int16_T re;
	int16_T im;
} cint16_T;

typedef struct {
	uint16_T re;
	uint16_T im;
} cuint16_T;

typedef struct {
	int32_T re;
	int32_T im;
} cint32_T;

typedef struct {
	uint32_T re;
	uint32_T im;
} cuint32_T;

typedef struct {
	int64_T re;
	int64_T im;
} cint64_T;

typedef struct {
	uint64_T re;
	uint64_T im;
} cuint64_T;

/*=======================================================================*
 * Min and Max:                                                          *
 *   int8_T, int16_T, int32_T     - signed 8, 16, or 32 bit integers     *
 *   uint8_T, uint16_T, uint32_T  - unsigned 8, 16, or 32 bit integers   *
 *=======================================================================*/
#define MAX_int8_T                     ((int8_T)(127))
#define MIN_int8_T                     ((int8_T)(-128))
#define MAX_uint8_T                    ((uint8_T)(255))
#define MIN_uint8_T                    ((uint8_T)(0))
#define MAX_int16_T                    ((int16_T)(32767))
#define MIN_int16_T                    ((int16_T)(-32768))
#define MAX_uint16_T                   ((uint16_T)(65535))
#define MIN_uint16_T                   ((uint16_T)(0))
#define MAX_int32_T                    ((int32_T)(2147483647))
#define MIN_int32_T                    ((int32_T)(-2147483647-1))
#define MAX_uint32_T                   ((uint32_T)(0xFFFFFFFFU))
#define MIN_uint32_T                   ((uint32_T)(0))
#define MAX_int64_T                    ((int64_T)(9223372036854775807LL))
#define MIN_int64_T                    ((int64_T)(-9223372036854775807LL-1LL))
#define MAX_uint64_T                   ((uint64_T)(0xFFFFFFFFFFFFFFFFULL))
#define MIN_uint64_T                   ((uint64_T)(0ULL))

 /* Logical type definitions */
#if !defined(__cplusplus) && !defined(__true_false_are_keywords)
#  ifndef false
#   define false                       (0U)
#  endif

#  ifndef true
#   define true                        (1U)
#  endif
#endif

/*
 * Maximum length of a MATLAB identifier (function/variable)
 * including the null-termination character. Referenced by
 * rt_logging.c and rt_matrx.c.
 */
#define TMW_NAME_LENGTH_MAX            64
#endif

extern real_T rtInf;
extern real_T rtMinusInf;
extern real_T rtNaN;
extern real32_T rtInfF;
extern real32_T rtMinusInfF;
extern real32_T rtNaNF;
extern void rt_InitInfAndNaN(size_t realSize);
extern boolean_T rtIsInf(real_T value);
extern boolean_T rtIsInfF(real32_T value);
extern boolean_T rtIsNaN(real_T value);
extern boolean_T rtIsNaNF(real32_T value);
typedef struct {
	struct {
		uint32_T wordH;
		uint32_T wordL;
	} words;
} BigEndianIEEEDouble;

typedef struct {
	struct {
		uint32_T wordL;
		uint32_T wordH;
	} words;
} LittleEndianIEEEDouble;

typedef struct {
	union {
		real32_T wordLreal;
		uint32_T wordLuint;
	} wordL;
} IEEESingle;


//devita_types.h
// Type Definitions
#ifndef struct_emxArray_real_T
#define struct_emxArray_real_T

struct emxArray_real_T
{
	double *data;
	int *size;
	int allocatedSize;
	int numDimensions;
	boolean_T canFreeData;
};

#endif   

// Function Declarations
extern void devita(double alpha, double beta, double eta, double epsilon, double
	epsilondot, double k, emxArray_real_T *theor_stress);
extern void devita_initialize();
extern void devita_terminate();
extern double st(double x, double alpha, double beta, double eta, double epsilon,
	double epsilondot, double k);

//devita_emxAPI.h
extern emxArray_real_T *emxCreateND_real_T(int numDimensions, int *size);
extern emxArray_real_T *emxCreateWrapperND_real_T(double *data, int
	numDimensions, int *size);
extern emxArray_real_T *emxCreateWrapper_real_T(double *data, int rows, int cols);
extern emxArray_real_T *emxCreate_real_T(int rows, int cols);
extern void emxDestroyArray_real_T(emxArray_real_T *emxArray);
extern void emxInitArray_real_T(emxArray_real_T **pEmxArray, int numDimensions);

//devita_emxutil.h
extern void emxEnsureCapacity_real_T(emxArray_real_T *emxArray, int oldNumel);
extern void emxEnsureCapacity_real_T1(emxArray_real_T *emxArray, int oldNumel);
extern void emxFree_real_T(emxArray_real_T **pEmxArray);
extern void emxInit_real_T(emxArray_real_T **pEmxArray, int numDimensions);
extern void emxInit_real_T1(emxArray_real_T **pEmxArray, int numDimensions);
 
// rtGetInf.h
extern real_T rtGetInf(void);
extern real32_T rtGetInfF(void);
extern real_T rtGetMinusInf(void);
extern real32_T rtGetMinusInfF(void);
 
// rtGetNaN.h
extern real_T rtGetNaN(void);
extern real32_T rtGetNaNF(void);


#ifdef SWIG
	#ifdef OSIMACTUATORS_API
		#undef OSIMACTUATORS_API
		#define OSIMACTUATORS_API
	#endif
#endif

namespace OpenSim {
	class GeometryPath;
	class NonLinearLigament : public Force {
		OpenSim_DECLARE_CONCRETE_OBJECT(NonLinearLigament, Force);
	public:

		// Properties declaration
		OpenSim_DECLARE_UNNAMED_PROPERTY(GeometryPath,
			"the set of points defining the path of the ligament");
		OpenSim_DECLARE_PROPERTY(resting_length, double,
			"resting length of the ligament");
		OpenSim_DECLARE_PROPERTY(pcsa_force, double,
			"force magnitude that scales the force-length curve");
		
		//OpenSim_DECLARE_PROPERTY(strain_rate, double, "strain rate");
		OpenSim_DECLARE_PROPERTY(eta_a, double, "eta a");
		OpenSim_DECLARE_PROPERTY(alphaa_a, double, "alpha a");
		OpenSim_DECLARE_PROPERTY(beta_a, double, "beta a");
		OpenSim_DECLARE_PROPERTY(ka_a, double, "ka a");
		OpenSim_DECLARE_PROPERTY(areaa_a, double, "area a");
		OpenSim_DECLARE_PROPERTY(max_strain, double, "maximum strain");
		//OpenSim_DECLARE_PROPERTY(ActiveForceLengthCurve1, Function, "Active Force length curve setter ");
		OpenSim_DECLARE_PROPERTY(DevitaModel, Function, "Devita Model");
		OpenSim_DECLARE_OPTIONAL_PROPERTY(force_length_curve, Function,"Function representing the force-length behavior of the ligament");
		
	public:

		//CONSTRUCTION

		NonLinearLigament();

		const GeometryPath& getGeometryPath() const
		{
			return get_GeometryPath();
		}
		GeometryPath& updGeometryPath()
		{
			return upd_GeometryPath();
		}
		virtual bool hasGeometryPath() const { return true; };
		virtual double getLength(const SimTK::State& s) const;
		virtual double getRestingLength() const
		{
			return get_resting_length();
		}
		virtual bool setRestingLength(double aRestingLength);
		virtual double getMaxIsometricForce() const
		{
			return get_pcsa_force();
		}
		virtual bool setMaxIsometricForce(double aMaxIsometricForce);


		// computed variables
		const double& getTension(const SimTK::State& s) const;

		//--------------------------------------------------------------------------
		// COMPUTATIONS
		//--------------------------------------------------------------------------
		virtual double computeMomentArm(const SimTK::State& s, Coordinate& aCoord) const;
		virtual void computeForce(const SimTK::State& s,
			SimTK::Vector_<SimTK::SpatialVec>& bodyForces,
			SimTK::Vector& generalizedForces) const;

		//double computePotentialEnergy(const SimTK::State & s) const;

		//--------------------------------------------------------------------------
		// SCALE
		//--------------------------------------------------------------------------
		virtual void preScale(const SimTK::State& s, const ScaleSet& aScaleSet);
		virtual void scale(const SimTK::State& s, const ScaleSet& aScaleSet);
		virtual void postScale(const SimTK::State& s, const ScaleSet& aScaleSet);


		//--------------------------------------------------------------------------
		// Display
		//--------------------------------------------------------------------------
		virtual const VisibleObject* getDisplayer() const;
		virtual void updateDisplayer(const SimTK::State& s) const;
		
		//virtual double getLengtheningSpeed(const SimTK::State& s) const;

		//double getStrainRate() const {return get_strain_rate(); };
		//void setStrainRate(double astrainrate);
		
		double getEtaA() const { return get_eta_a(); };
		virtual bool setEtaA(double eta);

		double getAlphaA() const { return get_alphaa_a(); };
		void setAlphaA(double alpha);

		double getBetaA() const { return get_beta_a(); };
		virtual bool setBetaA(double beta);

		double getKaA() const { return get_ka_a(); };
		virtual bool setKaA(double k);

		double getAreaA() const { return get_areaa_a(); };
		virtual bool setAreaA(double a);

		double getMaxStrain() const { return get_max_strain(); };
		virtual bool setMaxStrain(double a);


		
		//void setActiveForceLengthCurve1();
		
		
		//virtual bool setForceLengthCurve();
		virtual const Function& ForceLengthCurve() const { return get_force_length_curve(); }

		SimmSpline DevitaModel(const SimTK::State& s) const;
		virtual double getLengtheningSpeed(const SimTK::State& s) const;
		

	protected:
		virtual SimTK::Vec3 computePathColor(const SimTK::State& state) const;

		// Implement ModelComponent interface.
		/** Extension of parent class method; derived classes may extend further. **/
		void connectToModel(Model& aModel) OVERRIDE_11;
		/** Extension of parent class method; derived classes may extend further. **/
		void addToSystem(SimTK::MultibodySystem& system) const OVERRIDE_11;
		/** Extension of parent class method; derived classes may extend further. **/
		void realizeDynamics(const SimTK::State& state) const OVERRIDE_11;

		//Force reporting
		/**
		* Methods to query a Force for the value actually applied during simulation
		* The names of the quantities (column labels) is returned by this first function
		* getRecordLabels()
		*/
		OpenSim::Array<std::string> getRecordLabels() const {
			OpenSim::Array<std::string> labels("");
			labels.append(getName());
			return labels;
		}
		/**
		* Given SimTK::State object extract all the values necessary to report forces, application location
		* frame, etc. used in conjunction with getRecordLabels and should return same size Array
		*/
		OpenSim::Array<double> getRecordValues(const SimTK::State& state) const {
			OpenSim::Array<double> values(1);
			values.append(getTension(state));
			return values;
		}

	private:
		/** construct the new properties and set their default values */
		void constructProperties();
	};
}

#endif